﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using System.Threading;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace VSMDataflow
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            
        }

        private async void ActionButton_Click(object sender, RoutedEventArgs e)
        {
            ActionBlock<int> displayBlock = new ActionBlock<int>(async (x) =>
            {
                await Task.Delay(250);
                if (x % 25 == 0)
                    Data.Text = String.Empty;
                Data.Text += String.Format("{0}{1}", x, Environment.NewLine);
            },
            new ExecutionDataflowBlockOptions()
            {
                TaskScheduler = TaskScheduler.FromCurrentSynchronizationContext(),
                MaxDegreeOfParallelism = 4
            });

            for (int i = 0; i < 100; i++)
            {
                displayBlock.Post(i);
            }

            Data.Text = "Post completed!";

            await displayBlock.Completion;
        }

        private async void SquareActionButton_Click(object sender, RoutedEventArgs e)
        {
            var squareStringBuilder = new System.Text.StringBuilder();
            var squareBuilderBlock = new ActionBlock<int>((x) =>
            {
                int result = x * x;
                squareStringBuilder.AppendLine(result.ToString());
            }, new ExecutionDataflowBlockOptions()
            {
                MaxDegreeOfParallelism = 4
            });

            for (int i = 1; i <= 10; i++)
            {
                squareBuilderBlock.Post(i);
            }

            squareBuilderBlock.Complete();
            Data.Text = "Data posted!";

            await squareBuilderBlock.Completion;
            Data.Text = squareStringBuilder.ToString();
        }

        private async void TransformButton_Click(object sender, RoutedEventArgs e)
        {
            var squareRootBlock = new TransformBlock<double, double>(async x =>
                {
                    await Task.Delay(500);
                    return Math.Sqrt(x);
                });

            var numbers = Enumerable.Range(9, 25);

            foreach (var item in numbers)
            {
                await squareRootBlock.SendAsync(item);
            }

            Data.Text = "Data posted!";

            while (await squareRootBlock.OutputAvailableAsync())
            {
                var numReceived = await squareRootBlock.ReceiveAsync();
                Data.Text = String.Format("{0} is a result.", numReceived);
            }
        }
    }
}
